#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
#include "can.h"
#include "usart0.h"


void send_can_status(unsigned char status);
/*******************************************************************
CAN initialization:
Mailbox 0: Receive  --> interrupt
Mailbox 1: Transmit --> polling
*******************************************************************/
void can_init (void)
{
	CANGCON |= (1<<SWRES);						//reset CAN interface
	
	//reset all MObs
	for (unsigned char i=0; i<15; i++)
	{
		CANPAGE = (i<<4);						//select MOb
		CANCDMOB = 0;							//disable MOb
		CANSTMOB = 0;							//clear status
		CANIDT1 = 0;							//clear ID
		CANIDT2 = 0;
		CANIDT3 = 0;
		CANIDT4 = 0;
		CANIDM1 = 0;							//clear mask
		CANIDM2 = 0;
		CANIDM3 = 0;
		CANIDM4 = 0;
		for (unsigned char j=0; j<8; j++)
			CANMSG = 0;	//clear data
	}

	// bit timing -> datasheet 264 (check table)
	// 100Kbps 16MHz cpu-clk
        CANBT1 = 0x26;
        CANBT2 = 0x04;
        CANBT3 = 0x13;
	
	CANGCON = (1<<ENASTB);		//start CAN interface
	
	//configure MOb0
	CANPAGE = (0<<4);
/*	CANIDT1 = 0;			//ID
	CANIDT2 = 0;
	CANIDT3 = 0;
	CANIDT4 = 0;
	CANIDM1 = 0;			//get all messages
	CANIDM2 = 0;			//1 = check bit
	CANIDM3 = 0;			//0 = ignore bit
	CANIDM4 = (1<<IDEMSK);
*/	CANSTMOB = 0;
	CANCDMOB = (1<<CONMOB1) | (1<<IDE);	//reception, ext. ID
	
	//interrupts
	CANGIE = (1<<ENIT) | (1<<ENRX);	//enable receive interrupt
	CANIE1 = 0;
	CANIE2 = (1<<0);		//enable MOb0 interrupt
	CANSIT1 = 0;
	CANSIT2 = 0;
	
	while (!(CANGSTA & (1<<ENFG)));	//wait until module ready
}

/*******************************************************************
CAN transmission via mailbox 1 (polling)
*******************************************************************/
void can_tx (struct MOb msg)
{	
	//enable MOb1, auto increment index, start with index = 0
	CANPAGE = (1<<4);
	
	CANCDMOB = (1<<IDE) | (8<<DLC0);	//set IDE bit, length = 8
	
	msg.id <<= 3;				//write 29 Bit identifier
	CANIDT4 = (unsigned char) (msg.id&0xF8);
	CANIDT3 = (unsigned char) (msg.id>>8);
	CANIDT2 = (unsigned char) (msg.id>>16);
	CANIDT1 = (unsigned char) (msg.id>>24);
	
	for (unsigned char i=0; i<8; i++)	//put data in mailbox
		CANMSG = msg.data [i];
	
	CANCDMOB |= (1<<CONMOB0);		//enable transmission
        send_can_status(CANGSTA);
	usart0_transmit_string("Wait until send is finished\n");	
	while (!(CANSTMOB & (1<<TXOK)));	//wait until complete
	CANSTMOB &= ~(1<<TXOK);			//reset flag
	usart0_transmit_string("send is finished\n");
}

/*******************************************************************
CAN receive interrupt service routine (mailbox 0)
*******************************************************************/
SIGNAL (SIG_CAN_INTERRUPT1)			//receive interrupt
{
	usart0_transmit('R');
	unsigned char temp = CANPAGE;		//save CANPAGE
	struct MOb msg;
	
	CANPAGE = 0x00;				//select MOb0
	CANSTMOB &= ~(1<<RXOK);			//clear interrupt flag
	
	for (unsigned char i=0; i<8; i++)	//get data
		msg.data [i] = CANMSG;
	
	msg.id = 0;
	msg.id |= ((unsigned long) CANIDT1<<24);//get identifier
	msg.id |= ((unsigned long) CANIDT2<<16);
	msg.id |= ((unsigned long) CANIDT3<<8);
	msg.id |= (CANIDT4&0xF8);
	msg.id >>= 3;
	
	usart0_transmit(msg.data [0]);
	CANSTMOB = 0;
	CANCDMOB = (1<<CONMOB1) | (1<<IDE);	//reconfig mailbox
	
	CANPAGE = temp;				//restore CANPAGE
}

void send_can_status(unsigned char status){

        static unsigned char t=255;
        if (status != t) {
                t=status;
                usart0_transmit_string("\n\nCAN-Status\n");
                if (t & 1)
                        usart0_transmit_string("\tPassive Mode\n");

                if (t & (1<<1))
                        usart0_transmit_string("\tBus Off Mode\n");

                if (t & (1<<2))
                        usart0_transmit_string("\tCAN enabled\n");
                else
                        usart0_transmit_string("\tCAN disabled\n");

                if (t & (1<<3))
                        usart0_transmit_string("\tReceiver Busy\n");

                if (t & (1<<4))
                        usart0_transmit_string("\tTransmitter Busy\n");

                if (t & (1<<6))
                        usart0_transmit_string("\tOverload Frame Flag\n");
        }
        if (CANTEC!=0)
                usart0_transmit_string("\tCANTEC error\n");
        if (CANREC!=0)
                usart0_transmit_string("\tCANREC error\n");
}


int main(){
	struct MOb msg;
	msg.id=0x1;
	msg.data[0]=0x11;
	msg.data[1]=0x11;
	msg.data[2]=0x11;
	msg.data[3]=0x11;
	msg.data[4]=0x11;
	msg.data[5]=0x11;
	msg.data[6]=0x11;
	msg.data[7]=0x11;
//      usart0_init(51); // 19200 8N1 bei 16 MHz
        usart0_init(8); // 115200 8N1 bei 16 MHz
	usart0_transmit_string("\nHello World\n");
	can_init();
	sei();
	send_can_status(CANGSTA);
	can_tx(msg);	
	while(1)
		send_can_status(CANGSTA);
}


